## Subfamily Protein Architecture Labeling Engine Utility

##### (revised 18 September 2024)

### Background

The **Subfamily Protein Architecture Labeling Engine (SPARCLE)** Utility is also known as the **sparclbl command line utility**. It is an addition to the standalone version of Reverse Position-Specific BLAST (RPS-BLAST), and serves to functionally characterize and name the query protein based on its conserved domain architecture. A domain architecture is defined as the sequential order of conserved domains in a protein sequence.

The sparclbl command line utility is available, as of September 2019,
from the Conserved Domain Database (CDD) FTP site: 
https://ftp.ncbi.nih.gov/pub/mmdb/cdd/SparcleLabel/current

The program post-processes the output of RPS-BLAST. It reads the rpsblast output, inspects the identified conserved domains, and determines the domain architecture of the query sequence. By identifying the domain architecture of an unknown sequence, sparclbl can determine its functional group and assign a name accordingly. Read more in the SPARCLE help document: https://www.ncbi.nlm.nih.gov/sparcle.

### Download the software package

Precompiled binaries are available for x64-linux platforms. The latest version can be downloaded by the following command at the command prompt

```
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/SparcleLabel/current/SparcleLabel-x64-linux.tar.gz
--2024-09-18 12:14:37--  https://ftp.ncbi.nih.gov/pub/mmdb/cdd/SparcleLabel/current/SparcleLabel-x64-linux.tar.gz
Resolving ftp.ncbi.nih.gov (ftp.ncbi.nih.gov)... 10.10.26.7, 10.10.26.4, 10.10.26.6, ...
Connecting to ftp.ncbi.nih.gov (ftp.ncbi.nih.gov)|10.10.26.7|:443... connected.
HTTP request sent, awaiting response... 200 OK
Length: 7215509 (6.9M) [application/x-gzip]
Saving to: 'SparcleLabel-x64-linux.tar.gz'

SparcleLabel-x64-linux.tar.gz                 100%[=================================================================================================>]   6.88M  --.-KB/s    in 0.09s   

2024-09-18 12:14:37 (80.9 MB/s) - 'SparcleLabel-x64-linux.tar.gz' saved [7215509/7215509]

$ tar -xzf SparcleLabel-x64-linux.tar.gz
$ ls -l
drwxr-xr-x 4 shlu structure    4096 Sep 27 09:27 SparcleLabel-x64-linux
-rw-r--r-- 1 shlu structure 6384718 Sep 27 09:27 SparcleLabel-x64-linux.tar.gz
```

For other platforms, please download the source package:

```wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/SparcleLabel/current/SparcleLabel-src.tar.gz```

Unpack it and follow instructions in the `HOWTO.build.md` file to build the application on your computer.

### Download the BLAST binaries

The easiest way to do this is to run the script `utils/getblbins.sh` in the package. It automatically retrieves the last release from BLAST FTP site, decompress the tarball and create symbolic links and put everything in place.

```utils/getblbins.sh```


Under the hood, it does all the following:


Download the version for your platform from `https://ftp.ncbi.nih.gov/blast/executables/LATEST/` and untar it. At the time or writing, the latest package for linux is ncbi-blast-2.16.0+-x64-linux.tar.gz.
```
$ cd SparcleLabel-x64-linux
$ $ wget https://ftp.ncbi.nih.gov/blast/executables/LATEST/ncbi-blast-2.16.0+-x64-linux.tar.gz
--2024-09-18 12:37:59--  https://ftp.ncbi.nih.gov/blast/executables/LATEST/ncbi-blast-2.16.0+-x64-linux.tar.gz
Resolving ftp.ncbi.nih.gov (ftp.ncbi.nih.gov)... 10.10.26.4, 10.10.26.6, 10.10.26.13, ...
Connecting to ftp.ncbi.nih.gov (ftp.ncbi.nih.gov)|10.10.26.4|:443... connected.
HTTP request sent, awaiting response... 200 OK
Length: 257516053 (246M) [application/x-gzip]
Saving to: 'ncbi-blast-2.16.0+-x64-linux.tar.gz'

ncbi-blast-2.16.0+-x64-linux.tar.gz           100%[=================================================================================================>] 245.59M   403MB/s    in 0.6s    

2024-09-18 12:38:00 (403 MB/s) - 'ncbi-blast-2.16.0+-x64-linux.tar.gz' saved [257516053/257516053]

$ tar -xzf ncbi-blast-2.16.0+-x64-linux.tar.gz
$ ls -l
total 268612
-rw-r--r-- 1 shlu structure       530 Sep 18 10:55 HOWTO.run.md
drwxr-xr-x 2 shlu structure      4096 Sep 18 10:55 example
drwxr-xr-x 4 shlu structure      4096 Jun 25 14:31 ncbi-blast-2.16.0+
-rw-r--r-- 1 shlu structure 257516053 Jun 25 14:33 ncbi-blast-2.16.0+-x64-linux.tar.gz
-rw-r--r-- 1 shlu structure        27 Sep 18 10:55 precedences_sample.txt
-rwxr-xr-x 1 shlu structure  17490400 Sep 18 10:55 sparclbl
-rw-r--r-- 1 shlu structure       319 Sep 18 10:55 sparclbl.ini
-rwxr-xr-x 1 shlu structure     19489 Sep 18 10:55 sparclbl.sh
drwxr-xr-x 2 shlu structure      4096 Sep 18 10:55 utils
```
For convenience, make symbolic links of relevant programs to the current directory:

```
$ ln -s ncbi-blast-2.16.0+/bin/rpsblast ncbi-blast-2.16.0+/bin/rpstblastn ./
$ ls -l
total 268612
-rw-r--r-- 1 shlu structure       530 Sep 18 10:55 HOWTO.run.md
drwxr-xr-x 2 shlu structure      4096 Sep 18 10:55 example
drwxr-xr-x 4 shlu structure      4096 Jun 25 14:31 ncbi-blast-2.16.0+
-rw-r--r-- 1 shlu structure 257516053 Jun 25 14:33 ncbi-blast-2.16.0+-x64-linux.tar.gz
-rw-r--r-- 1 shlu structure        27 Sep 18 10:55 precedences_sample.txt
lrwxrwxrwx 1 shlu structure        31 Sep 18 12:41 rpsblast -> ncbi-blast-2.16.0+/bin/rpsblast
lrwxrwxrwx 1 shlu structure        33 Sep 18 12:41 rpstblastn -> ncbi-blast-2.16.0+/bin/rpstblastn
-rwxr-xr-x 1 shlu structure  17490400 Sep 18 10:55 sparclbl
-rw-r--r-- 1 shlu structure       319 Sep 18 10:55 sparclbl.ini
-rwxr-xr-x 1 shlu structure     19489 Sep 18 10:55 sparclbl.sh
drwxr-xr-x 2 shlu structure      4096 Sep 18 10:55 utils
```

### Download database and data files

Again, it is the easiest to just run the provided script
```utils/getcdddata.sh```

It automatically retrieve all needed database and data files, decompress them and put them to the right place. If the script cannot run on your platform, then you need to to the following manual steps:

Create a `db` directory and put the database files there:

```
$ mkdir ./db
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/little_endian/Cdd_LE.tar.gz && tar -xzf Cdd_LE.tar.gz -C ./db && rm -f Cdd_LE.tar.gz
--2024-09-18 13:05:09--  https://ftp.ncbi.nih.gov/pub/mmdb/cdd/little_endian/Cdd_LE.tar.gz
Resolving ftp.ncbi.nih.gov (ftp.ncbi.nih.gov)... 10.10.26.13, 10.10.26.5, 10.10.26.6, ...
Connecting to ftp.ncbi.nih.gov (ftp.ncbi.nih.gov)|10.10.26.13|:443... connected.
HTTP request sent, awaiting response... 200 OK
Length: 1745781939 (1.6G) [application/x-gzip]
Saving to: 'Cdd_LE.tar.gz'

Cdd_LE.tar.gz                                 100%[=================================================================================================>]   1.62G   271MB/s    in 6.0s    

2024-09-18 13:05:16 (276 MB/s) - 'Cdd_LE.tar.gz' saved [1745781939/1745781939]
$ ls -l db/
total 3727680
-rw-r--r-- 1 shlu structure     42320 Mar 29 09:44 Cdd.00.aux
-rw-r--r-- 1 shlu structure  82649428 Mar 29 09:44 Cdd.00.freq
-rw-r--r-- 1 shlu structure  19650096 Mar 29 09:44 Cdd.00.loo
-rw-r--r-- 1 shlu structure     94208 Mar 29 09:44 Cdd.00.pdb
-rw-r--r-- 1 shlu structure    483190 Mar 29 09:44 Cdd.00.phr
-rw-r--r-- 1 shlu structure     20072 Mar 29 09:44 Cdd.00.pin
-rw-r--r-- 1 shlu structure     47470 Mar 29 09:44 Cdd.00.pos
-rw-r--r-- 1 shlu structure     29984 Mar 29 09:44 Cdd.00.pot
-rw-r--r-- 1 shlu structure    737853 Mar 29 09:44 Cdd.00.psq
-rw-r--r-- 1 shlu structure     16384 Mar 29 09:44 Cdd.00.ptf
-rw-r--r-- 1 shlu structure     10040 Mar 29 09:44 Cdd.00.pto
-rw-r--r-- 1 shlu structure  82649428 Mar 29 09:44 Cdd.00.rps
   ...........
```
The "Cdd" database (filename: Cdd_LE.tar.gz) is sufficient for almost all searches; it is a superset including NCBI-curated domains and data imported from Pfam, SMART, COG, PRK, and TIGRFAMs. It is also the default database for the online CD-Search system.

However, the following databases are also available and can be obtained, if desired, in similar ways:
```
Cdd_NCBI_LE.tar.gz
Cog_LE.tar.gz
Kog_LE.tar.gz
Prk_LE.tar.gz
Smart_LE.tar.gz
Tigr_LE.tar.gz
```
For more information regarding the RPSBLAST databases, read the documentation at
```
https://www.ncbi.nlm.nih.gov/Structure/cdd/cdd_help.shtml#RPSBSearchDb
```

The sparclbl program will need a set of domain-annotation data files to function. Download them from the ftp site and put them under "data" directory:

```
$ mkdir ./data
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/cddid.tbl.gz -O ./data/cddid.tbl.gz && gzip -d ./data/cddid.tbl.gz
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/cdtrack.txt -O ./data/cdtrack.txt
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/family_superfamily_links -O ./data/family_superfamily_links
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/cddannot.dat.gz -O ./data/cddannot.dat.gz && gzip -d ./data/cddannot.dat.gz
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/cddannot_generic.dat.gz -O ./data/cddannot_generic.dat.gz && gzip -d ./data/cddannot_generic.dat.gz
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/bitscore_specific.txt -O ./data/bitscore_specific.txt
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/specific_arch.txt -O ./data/specific_arch.txt
$ wget https://ftp.ncbi.nih.gov/pub/mmdb/cdd/superfamily_arch.txt -O ./data/superfamily_arch.txt

```

Now that everything is in place, you should have the following directory structure:
```
.
|-- HOWTO.run.md
|-- data
|   |-- bitscore_specific.txt
|   |-- cddannot.dat
|   |-- cddannot_generic.dat
|   |-- cddid.tbl
|   |-- cdtrack.txt
|   |-- family_superfamily_links
|   |-- specific_arch.txt
|   |-- superfamily_arch
|   `-- superfamily_arch.txt
|-- db
|   |-- Cdd.00.aux
|   |-- Cdd.00.freq
|   |-- Cdd.00.loo
|   |-- Cdd.00.pdb
|   |-- Cdd.00.phr
|   |-- Cdd.00.pin
|   |-- Cdd.00.pos
|   |-- Cdd.00.pot
|   |-- Cdd.00.psq
|   |-- Cdd.00.ptf
|   |-- Cdd.00.pto
|   |-- Cdd.00.rps
|   .........................
|   |-- 
|   `-- Cdd.pal
|-- example
|   `-- Ecoli.fa
|-- ncbi-blast-2.16.0+
|   |-- BLAST_PRIVACY
|   |-- ChangeLog
|   |-- LICENSE
|   |-- README
|   |-- bin
|   |   .........................
|   |   |-- rpsblast
|   |   |-- rpstblastn
|   |   .........................
|   |-- doc
|   |   `-- README.txt
|   `-- ncbi_package_info
|-- ncbi-blast-2.16.0+-x64-linux.tar.gz
|-- precedences_sample.txt
|-- rpsblast -> ncbi-blast-2.16.0+/bin/rpsblast
|-- rpstblastn -> ncbi-blast-2.16.0+/bin/rpstblastn
|-- sparclbl
|-- sparclbl.ini
|-- sparclbl.sh
`-- utils
    |-- getblbins.sh
    `-- getcdddata.sh
```
The software is now ready to run.

### Data format

Both `rpsblast` and `rpstblastn` support FASTA formatted sequences. Put your FASTA sequences in a file as input to these two programs. The package included an example FASTA sequence file `example/Ecoi.fa`.

The output of `rpsblast` and `rpstblastn` programs will serve as the input of the `sparclbl` program. However, `sparclbl` only processes the asn1 formatted Blast Archive object, because it contains all the information necessary for sparclbl to function. Therefore, the output format of the `rpsblast` and `rpstblastn` should be set to 11:

```
$ ./rpsblast -parse_deflines -query example/Ecoli.fa -db ./db/Cdd -evalue 0.01 -outfmt 11 -out Ecoli_annot.asn
```

Blast programs may run for a few minutes, depends on the machine. When it is done, the file "Ecoli_annot.asn" generated by rpsblast can then be processed by sparclbl in to assign names to sequences. The output of `sparclbl` is in `xml` format:
```
$ ./sparclbl -p precedences_sample.txt -o Ecoli_names.xml Ecoli_annot.asn
```

or in one line of command using pipe:
```
./rpsblast -parse_deflines -query example/Ecoli.fa -db ./db/Cdd -evalue 0.01 -outfmt 11 | ./sparclbl -p precedences_sample.txt > Ecoli_names.xml
```

For last number of sequences in multiple FASTA files, batch mode may be preferred. In this mode, multiple instances of `rpsblast` or `rpstblastn` are launched due to the fact that `sparclbl` runs much faster than BLAST. `sparclbl` is launched and run in background, processing any output file from BLAST as soon as they are produced. All named sequences are collected in a single `.xml` file.

This mode can be launched using the included script `sparclbl.sh`:

```./sparclbl.sh -q "path/to/fasta-files/*.fa" -p precedences_sample.txt -d ./data -b ./db/Cdd -o manyseqs.xml```

Run `./sparclbl.sh -h` for detailed usage.


### The naming levels

The sparclbl utility identifies domain architecture in query sequence and assigns a name to the protein. Specifically, sparclbl analyzes the conserved domains on each query sequence and determines whether they represents a valid domain architecture that exists in a known protein sequence. If so, sparclbl assigns the name of that domain architecture to the query sequence. 

Three tiers of domain architecture names exist: "CDD", "CDDauto" and "CDDdom", and each one represents a different level of confidence, as described below. 

Sparclbl therefore requires a precedence-definition file (i.e., the -p switch is mandatory) to define a priority value
to each domain architecture name based on the method that was used to generate the name. 

The supplied file "precedences_sample.txt" can be used for this purpose directly, as shown in the sample code above.

1. **CDD level**. Domain architecture names that are tagged "CDD" are manually curated and thus have the highest level of confidence. As a result, they are given the highest priority in the precedence-definition file, and take precedence when assigning names to query proteins. (Note: The names in this set are also used by the NCBI Prokaryotic Genome Annotation Pipeline (PGAP), which integrates multiple naming algorithms, including SPARCLE's curated "CDD" set.)

2. **CDDAuto level**. Domain architecture names that are tagged "CDDAuto" are computer generated by analysing the descriptive definition of known proteins bearing this architecture. This type of name tends to have a more proper format than the "CDDdom" architectures, and therefore have a higher priority over "CDDdom" names in the precedence-definition file.

3. **CDDdom level**. Domain architecture names that are tagged "CDDdom" are also computer generated; they are are constructed in an automated way from the names of conserved domains that compose the architecture. Such architecture names, while more informative than "hypothetical protein", do not necessarily hint at a protein's function. This is because architecture names that are based on domain composition are not curated in any way. "CDDdom" names therefore have the lowest priority in the precedence-definition file.

Additional information about the three levels of conserved domain architectures, along with examples, are provided in the online SPARCLE help document: `https://www.ncbi.nlm.nih.gov/Structure/sparcle/docs/sparcle_help.html#DataProcessingReviewLevel`

### Sparclbl Output

The output from `sparclbl` is an `.xml` file with the following structure:
```
<?xml version="1.0" encoding="utf-8" standalone="no"?>
<!DOCTYPE NameAssignment SYSTEM "http://www.ncbi.nlm.nih.gov/gpipe/protein_naming/cluster_assignment.dtd">
<NameAssignment>
	<Protein seq-id="ref|WP_000002310.1">
		<AssignedName>alpha-D-ribose 1-methylphosphonate 5-phosphate C-P-lyase PhnJ</AssignedName>
		<Support>
			<SPARCLE>
				<ArchId>10007623</ArchId>
				<Name>alpha-D-ribose 1-methylphosphonate 5-phosphate C-P-lyase PhnJ</Name>
				<Label>alpha-D-ribose 1-methylphosphonate 5-phosphate C-P-lyase PhnJ is part of the C-P lyase complex, which is responsible for processing phophonates into usable phosphate</Label>
				<SpecificArch>COG3627</SpecificArch>
			</SPARCLE>
		</Support>
		<Precedence>
			<evidence_source>CDD</evidence_source>
			<confidence>60</confidence>
		</Precedence>
	</Protein>
	<Protein seq-id="ref|WP_000002542.1">
	    ... ... ...
	</Protein>
	... ... ...
</NameAssignment>

```

Each protein has a `seq-id`, which is from the definition line of the FASTA. `<AssignedName>` is the name assigned to the protein. `<Support>` node contains the reason/evidence to support this assignment. For `sparclbl`, this node always contains the architecture that have been identified on the protein sequence. The `<Name>` of the architecture is assigned to the protein (thus the same as `<AssignedName>`). `<Label>` is a more descriptive name for the architecture and its associated functionality. 

`<Precedence>` gives the tag of naming method(`<evidence_source>`) and confidence value, exactly as specified in precedence-definition file.

